import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class Collision here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Collision  extends Actor
{

    private int WIDTH;
    private int HEIGHT;
    
    /**
     * Constructor 
     * defines the dimensions of the window
     */
    public Collision(int x, int y){
        WIDTH = x;
        HEIGHT = y;
    }
    
    public void doPhysics(GameObject g){
        fixCollision(g);
        if(g instanceof Player){    
            fixSpecialCollision((Player)g);
        }
    }

    public void setDimensions(int x, int y){
        WIDTH = x;
        HEIGHT = y;
    }  
    
    private boolean spinnerFix(GameObject p){
        if(p instanceof Spinner){
            if( ((Spinner)p).getDirection() == Direction.RIGHT || ((Spinner)p).getDirection() == Direction.LEFT){
                return false;
            }
        }
        return true;
    }

    /**
     * Checks collision for the player
     */
    public void fixCollision(GameObject p)
    {
        int x1 = (int)Math.round(p.getExactX());
        int y1 = (int)Math.round(p.getExactY());
        int x2 = (int)Math.round(p.getOldX());
        int y2 = (int)Math.round(p.getOldY());
        boolean fix = collide(p);
        boolean favorY = spinnerFix(p);
        if(fix){  
            p.setUpdatePosition(false);
        }

        if(favorY){
            while(collide(p)){  
                if(y1 < y2){
                   y1++;
                }else if(y1 > y2){           
                   y1--;
                }else if (y1 == y2){
                    if(x1 < x2){
                        x1++;
                    }else if(x1 > x2){
                        x1--;
                    }
                }
                p.fixX(x1);
                p.fixY(y1);
                p.setLocation(x1,y1);
            }
        }else{
            while(collide(p)){  
                if(x1 < x2){
                   x1++;
                }else if(x1 > x2){           
                   x1--;
                }else if (x1 == x2){
                    if(y1 < y2){
                        y1++;
                    }else if(y1 > y2){
                        y1--;
                    }
                }
                p.fixX(x1);
                p.fixY(y1);
                p.setLocation(x1,y1);
            }
        }
        
        if(fix){
            p.setExactX(x1);
            p.setExactY(y1);          
        }        
        updateWallState(p); //does special action depending on object type      
        p.setUpdatePosition(true);   
    }   

    public void fixSpecialCollision(Player p)
    {
        int x1 = (int)Math.round(p.getExactX());
        int y1 = (int)Math.round(p.getExactY());
        int x2 = (int)Math.round(p.getOldX());
        int y2 = (int)Math.round(p.getOldY());
        boolean specialFix = false;
        boolean vert = (y1==y2);
        
        if(collidePassableWall(p, specialFix) && !vert){ 
            p.setUpdatePosition(false);
            specialFix = true;
        }
        
        while(collidePassableWall(p, specialFix)){  
            if(x1 < x2){
               x1++;
            }else if(x1 > x2){           
               x1--;
            }else if (x1 == x2){
                if(y1 < y2){
                    y1++;
                }else if(y1 > y2){
                    y1--;
                }
            }
            p.fixX(x1);
            p.fixY(y1);
            p.setLocation(x1,y1);
        }        
        
        if(specialFix){
            p.setExactX(x1);
            p.setExactY(y1);
        }
        wallStateSpecial(p);
        p.setUpdatePosition(true);   
    }  
    
    private boolean collidePassableWall(Player p, boolean prevFix)     //prevFix forces function to return true if it is continuing to fix a collision
    {
        int w = p.getImageWidth()/2;
        int h = p.getImageHeight()/2;
        int x = (int)Math.round(p.getExactX());
        int y = (int)Math.round(p.getExactY());
        
        boolean flipped = p.getState(State.FLIPPED);
        boolean prevWall = p.getState(State.INWALL);      //player was in this wall last frame
        boolean inWall = false;
        boolean fix = false;
        
        
        for(int i=x-w; i<=x+w; i++){
            for(int j=y-h; j<=y+h; j++){
                   if(oneWay(i,j) || oneWayDown(i,j)){
                       inWall = true;
                   }                   
            }
        }        
        
        
        if( oneWay(x, y-h) ){                //top center
            if(flipped){fix = true;}
        }else if( oneWay(x-w, y-h) ){        //top left
            if(flipped){fix = true;}
        }else if( oneWay(x+w, y-h) ){        //top right
            if(flipped){fix = true;}   
        }else if( oneWay(x, y+h) ){         //bottom center
            if(!flipped){fix = true;}
        }else if( oneWay(x-w, y+h) ){        //bottom left
            if(!flipped){fix = true;}
        }else if( oneWay(x+w, y+h) ){        //bottom right
            if(!flipped){fix = true;}
        }      

        if( oneWayDown(x, y-h)){                //top center
            fix = true;
        }else if( oneWayDown(x-w, y-h)){        //top left
            fix = true;
        }else if( oneWayDown(x+w, y-h)){        //top right
            fix = true;
        }else if( oneWayDown(x, y+h)){         //bottom center
            if(!flipped){fix = true;}
        }else if( oneWayDown(x-w, y+h)){        //bottom left
            if(!flipped){fix = true;}
        }else if( oneWayDown(x+w, y+h)){        //bottom right
            if(!flipped){fix = true;}
        }      
        
        p.setState(State.INWALL, inWall);     
        if((!prevWall || prevFix) && inWall && fix){    //if player was previously in wall, it is assumed the player was passing through and should not be stopped
            return true;
        }else{
            return false;
        }
    }  

    private void wallStateSpecial(Player p)
    {
        int w = p.getImageWidth()/2;
        int h = p.getImageHeight()/2;
        int x = (int)Math.round(p.getExactX());
        int y = (int)Math.round(p.getExactY());    
        boolean flipped = p.getState(State.FLIPPED);
        boolean oneWayCeiling = false;
        boolean oneWayDownCeiling = false;

        if( oneWay(x  , Math.min(y+h+1, HEIGHT))  ||       //check one pixel below player
            oneWay(x+w, Math.min(y+h+1, HEIGHT))  ||
            oneWay(x-w, Math.min(y+h+1, HEIGHT))  ||
            oneWayDown(x  , Math.min(y+h+1, HEIGHT))  ||       //check one pixel below player
            oneWayDown(x+w, Math.min(y+h+1, HEIGHT))  ||
            oneWayDown(x-w, Math.min(y+h+1, HEIGHT)) 

            ){
        
                if(!flipped && p.getState(State.INWALL) == false){
                    p.setState(State.GROUNDSP , true);
                    p.setDy(0);
                }else{
                    p.setState(State.GROUNDSP , false);                    
                }
                   
        }else{
            p.setState(State.GROUNDSP , false);                    
        }
        
        if( oneWay(x  , Math.max(y-h-1, 0))  ||            //check one pixel above player
            oneWay(x+w, Math.max(y-h-1, 0))  ||
            oneWay(x-w, Math.max(y-h-1, 0))
                                                ){
        
                if(flipped && p.getState(State.INWALL) == false){
                    p.setState(State.CEILINGSP , true);                    
                    p.setDy(0);                    
                    oneWayCeiling = true;
                }else{
                    p.setState(State.CEILINGSP , false);                                        
                }
        }else{
            p.setState(State.CEILINGSP , false);                                        
        }     

        if( oneWayDown(x  , Math.max(y-h-1, 0))  ||            //check one pixel above player
            oneWayDown(x+w, Math.max(y-h-1, 0))  ||
            oneWayDown(x-w, Math.max(y-h-1, 0))
                                                ){
        
                if(p.getState(State.INWALL) == false){
                    p.setState(State.CEILINGSP , true);                    
                    p.setDy(0);            
                    oneWayDownCeiling = true;        
                }else{
                    p.setState(State.CEILINGSP , false);                                        
                }
        }else{
            p.setState(State.CEILINGSP , false);                                        
        } 

        if(oneWayCeiling || oneWayDownCeiling){
            p.setState(State.CEILINGSP , true);  
            p.setDy(0);   
        }else{
            p.setState(State.CEILINGSP , false);
        }

    }

    public boolean collide(GameObject g){
        if(g instanceof Player){
            return collidePlayer((Player)g);    
        }else if(g instanceof Enemy){
            return collideEnemy((Enemy)g);
        }
            return false;
    }
    
    private boolean collidePlayer(Player p)
    {
        int w = p.getImageWidth()/2;
        int h = p.getImageHeight()/2;
        int x = (int)Math.round(p.getExactX());
        int y = (int)Math.round(p.getExactY());
        
        boolean flipped = p.getState(State.FLIPPED);
        boolean inWall = false;
        
        for(int i=x-w; i<=x+w; i+=w){
            for(int j=y-h; j<=y+h; j+=h){
                if(i!=x && j!=y){   //skip the center of the player
                    if( solid(i,j) || solidNoPierce(i,j)){
                        inWall = true;
                    }
                    if( spike(i,j) ){
                        p.setState(State.HIT, true);
                    }
                }
            }
        }            
        return inWall;
    } 
    
    private boolean collideEnemy(Enemy p)
    {
        int w = p.getImage().getWidth()/2;
        int h = p.getImage().getHeight()/2;
        int x = (int)Math.round(p.getExactX());
        int y = (int)Math.round(p.getExactY());
        
        boolean inWall = false;
        
        for(int i=x-w; i<=x+w; i+=w){
            for(int j=y-h; j<=y+h; j+=h){
                if(i!=x && j!=y){   //skip the center
                    if( anySolid(i,j) ){
                        inWall = true;
                    }
                }
            }
        }  
           
        return inWall;
    }       
    
    public void updateWallState(GameObject g){
        if(g instanceof Player){
            wallStatePlayer((Player)g);
        }else if(g instanceof Walker){
            wallStateWalker((Walker)g);
        }else if(g instanceof Spinner){
            wallStateSpinner((Spinner)g);
        }
    }

    public void wallStatePlayer(Player p)
    {
        int w = p.getImageWidth()/2;
        int h = p.getImageHeight()/2;
        int x = (int)Math.round(p.getExactX());
        int y = (int)Math.round(p.getExactY());    
        boolean flipped = p.getState(State.FLIPPED);
        
        if( endSection(Math.min(x+w+1, WIDTH) , y  )||
            endSection(Math.min(x+w+1, WIDTH) , y+h)||
            endSection(Math.min(x+w+1, WIDTH) , y-h)){
        
                WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0)); 
                stats.setNextSectionFlag(true);
                return;
        }
        
        if( solid(x  , Math.min(y+h+1, HEIGHT)) ||
            solid(x+w, Math.min(y+h+1, HEIGHT)) ||
            solid(x-w, Math.min(y+h+1, HEIGHT)) ||
            solidNoPierce(x  , Math.min(y+h+1, HEIGHT)) ||
            solidNoPierce(x+w, Math.min(y+h+1, HEIGHT)) ||
            solidNoPierce(x-w, Math.min(y+h+1, HEIGHT))
                                                        ){
        
                p.setState(State.GROUND , true);
                p.setDy(0);
        }else{
                p.setState(State.GROUND , false);            
        }
        
        if( solid(x  , Math.max(y-h-1, 0)) ||
            solid(x+w, Math.max(y-h-1, 0)) ||
            solid(x-w, Math.max(y-h-1, 0)) ||
            solidNoPierce(x  , Math.max(y-h-1, 0)) ||
            solidNoPierce(x+w, Math.max(y-h-1, 0)) ||
            solidNoPierce(x-w, Math.max(y-h-1, 0)) 

                                                    ){
        
                p.setState(State.CEILING , true);
                p.setDy(0);                
        }else{
                p.setState(State.CEILING , false);            
        }
        
        if( solid(Math.min(x+w+1, WIDTH) , y  ) ||
            solid(Math.min(x+w+1, WIDTH) , y+h) ||
            solid(Math.min(x+w+1, WIDTH) , y-h) ||
            solidNoPierce(Math.min(x+w+1, WIDTH) , y  ) ||
            solidNoPierce(Math.min(x+w+1, WIDTH) , y+h) ||
            solidNoPierce(Math.min(x+w+1, WIDTH) , y-h)
                                                    ){
                p.setState(State.RWALL , true);
        }else{
                p.setState(State.RWALL , false);            
        }
        
        if( solid(Math.max(x-w-1,0) , y  ) ||
            solid(Math.max(x-w-1,0) , y+h) ||
            solid(Math.max(x-w-1,0) , y-h) ||
            solidNoPierce(Math.max(x-w-1,0) , y  ) ||
            solidNoPierce(Math.max(x-w-1,0) , y+h) ||
            solidNoPierce(Math.max(x-w-1,0) , y-h)
                                                ){
                p.setState(State.LWALL , true);
        }else{
                p.setState(State.LWALL , false);            
        }          
    }  
    
    private void wallStateWalker(Walker p)
    {
        int w = p.getImage().getWidth()/2;
        int h = p.getImage().getHeight()/2;
        int x = (int)Math.round(p.getExactX());
        int y = (int)Math.round(p.getExactY());
        boolean flipped = p.getState(WalkerState.FLIPPED);
        
        if( anySolid(x  , Math.min(y+h+1, HEIGHT)) ||       //checks for ground
            anySolid(x+w, Math.min(y+h+1, HEIGHT)) ||
            anySolid(x-w, Math.min(y+h+1, HEIGHT))){
                p.setState(WalkerState.GROUND , true);
                if(!flipped){
                    p.setState(WalkerState.CANFLIP, true);
                }
        }else{
                p.setState(WalkerState.GROUND , false);
        }

        if( anySolid(x  , Math.max(y-h-1, 0)) ||            //checks for ceiling
            anySolid(x+w, Math.max(y-h-1, 0)) ||
            anySolid(x-w, Math.max(y-h-1, 0))){
                p.setState(WalkerState.CEILING , true);
                if(flipped){
                    p.setState(WalkerState.CANFLIP, true);
                }                
        }else{
                p.setState(WalkerState.CEILING , false);            
        }
        
        if( anySolid(Math.min(x+w+1, WIDTH) , y  )||        //checks for wall on right and flags for turnaround
            anySolid(Math.min(x+w+1, WIDTH) , y+h)||
            anySolid(Math.min(x+w+1, WIDTH) , y-h)){
        
                p.setState(WalkerState.TURN , true);
        }          
        
        if( anySolid(Math.max(x-w-1,0) , y  ) ||
            anySolid(Math.max(x-w-1,0) , y+h) ||
            anySolid(Math.max(x-w-1,0) , y-h)){
        
                p.setState(WalkerState.TURN , true);
        }
        
        if(!p.getState(WalkerState.FLIPPED) && p.getState(WalkerState.GROUND) && p.getState(WalkerState.RIGHT) && !anySolid(Math.min(x+w+1, WIDTH) , y+h+1)){                                                   //checks for 
                p.setState(WalkerState.TURN , true);               
        }
        if(p.getState(WalkerState.FLIPPED) && p.getState(WalkerState.CEILING) && p.getState(WalkerState.RIGHT) && !anySolid(Math.min(x+w+1, WIDTH) , y-h-1)){                                                   //checks for 
                p.setState(WalkerState.TURN , true);               
        }      
        if(!p.getState(WalkerState.FLIPPED) && p.getState(WalkerState.GROUND) && p.getState(WalkerState.LEFT) && !anySolid(Math.min(x-w-1, WIDTH) , y+h+1)){                                                   //checks for 
                p.setState(WalkerState.TURN , true);               
        }
        if(p.getState(WalkerState.FLIPPED) && p.getState(WalkerState.CEILING) && p.getState(WalkerState.LEFT) && !anySolid(Math.min(x-w-1, WIDTH) , y-h-1)){                                                   //checks for 
                p.setState(WalkerState.TURN , true);               
        }          
        
    }    
    
    private void wallStateSpinner(Spinner p)
    {
        int w = p.getImage().getWidth()/2;
        int h = p.getImage().getHeight()/2;
        int x = (int)Math.round(p.getExactX());
        int y = (int)Math.round(p.getExactY());
        
        if( anySolid(x  , Math.min(y+h+1, HEIGHT)) ||       //checks for ground
            anySolid(x+w, Math.min(y+h+1, HEIGHT)) ||
            anySolid(x-w, Math.min(y+h+1, HEIGHT))){
                p.setState(SpinnerState.GROUND , true);
        }else{
                p.setState(SpinnerState.GROUND , false);
        }

        if( anySolid(x  , Math.max(y-h-1, 0)) ||            //checks for ceiling
            anySolid(x+w, Math.max(y-h-1, 0)) ||
            anySolid(x-w, Math.max(y-h-1, 0))){
                p.setState(SpinnerState.CEILING , true);              
        }else{
                p.setState(SpinnerState.CEILING , false);            
        }
        
        if( anySolid(Math.min(x+w+1, WIDTH) , y  )||        //checks for wall on right
            anySolid(Math.min(x+w+1, WIDTH) , y+h)||
            anySolid(Math.min(x+w+1, WIDTH) , y-h)){
        
                p.setState(SpinnerState.RWALL , true);
        }else{
                p.setState(SpinnerState.RWALL , false);
        }          
        
        if( anySolid(Math.max(x-w-1,0) , y  ) ||        //checks for wall on left
            anySolid(Math.max(x-w-1,0) , y+h) ||
            anySolid(Math.max(x-w-1,0) , y-h)){

                p.setState(SpinnerState.LWALL , true);
        }else{
                p.setState(SpinnerState.LWALL , false);
        }
    }

    public boolean anySolid(int x, int y)
    {
        return (solid(x,y) || solidNoPierce(x,y) || oneWay(x,y) || oneWayDown(x,y) || spike(x,y) || endSection(x,y));
    }

    public boolean solid(int x, int y)
    {
        WorldStats stats = ((WorldStats)(getWorld().getObjects(WorldStats.class).get(0)));
        Level level = stats.getCurrentLevel();
        return (level.getCurrentLogicalImage(stats.getSection())).getColorAt(x, y).equals(Logic.SOLID);
    }

    public boolean solidNoPierce(int x, int y)
    {
        WorldStats stats = ((WorldStats)(getWorld().getObjects(WorldStats.class).get(0)));
        Level level = stats.getCurrentLevel();
        return (level.getCurrentLogicalImage(stats.getSection())).getColorAt(x, y).equals(Logic.SOLID_NO_PIERCE);
    }
    
    public boolean oneWay(int x, int y)
    {        
        WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));        
        Level level = stats.getCurrentLevel();        
        return (level.getCurrentLogicalImage(stats.getSection())).getColorAt(x, y).equals(Logic.ONE_WAY_BOTH_DIR);
    }    

    public boolean oneWayDown(int x, int y)
    {        
        WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));        
        Level level = stats.getCurrentLevel();
        return (level.getCurrentLogicalImage(stats.getSection())).getColorAt(x, y).equals(Logic.ONE_WAY_DOWN);
    }   
    
    public boolean endSection(int x, int y)
    {        
        WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));        
        Level level = stats.getCurrentLevel();
        return (level.getCurrentLogicalImage(stats.getSection())).getColorAt(x, y).equals(Logic.END_SECTION);
    }        

    public boolean spike(int x, int y)
    {        
        WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));        
        Level level = stats.getCurrentLevel();
        return (level.getCurrentLogicalImage(stats.getSection())).getColorAt(x, y).equals(Logic.SPIKE);
    }  
}
