import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.List;
/**
 * Write a description of class GameWorld here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class GameWorld  extends World
{
    private int timer;
    private boolean firstTime;
    private Runtime r = Runtime.getRuntime(); 
    private static Collision collision;
    private static Player player;
    private static WorldStats stats;
    private static GreenfootImage transition;
    private static MusicController music;

    private static final GreenfootImage gameOver = new GreenfootImage("images/gameOver.png"); 
    private static final GreenfootImage gameStart = new GreenfootImage("images/Title.png");
    /**
     * Constructor for objects of class GameWorld.
     * 
     */
    public GameWorld()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(256, 225, 1);
        Greenfoot.setSpeed(50);
        firstTime = true;
        //xScroll = 0;
        //yScroll = 0;
        timer = 1;
        setPaintOrder(ShooterBullet.class, Player.class, PlayerBullet.class, Boss1.class, Boss1Bullet.class, Boss1Overlay.class, Boss1Part.class); 
        stats = new WorldStats();        
        music = new MusicController();       
        stats.getImage().setTransparency(0);         
        music.getImage().setTransparency(0);  
        collision = new Collision(stats.getCurrentLevel().getCurrentLogicalImage(stats.getSection()).getWidth(), stats.getCurrentLevel().getCurrentLogicalImage(stats.getSection()).getHeight());
        collision.getImage().setTransparency(0);
    }
    
    public void started()
    {
        if(firstTime){
            player = new Player(25, 152);
            player.getImage().setTransparency(0);
            player.setUpdatePosition(false);
            addObject(player, 25, 152);
            addObject(music,0,0);
            addObject(stats,0,0);    
            addObject(collision,0,0);
            firstTime = false;
        }else{
            music.playPause();
            music.setMusicStart(false);    
        }
    }    
    
    public void stopped(){
        music.stopMusic();
        music.playPause();        
    }
    
    public void act()
    { 
        if(stats.getGameStartFlag() == false){
            gameStart();
            return;
        }
        if (stats.getGameOverFlag()){
            gameOver();
            return;
        }
        timer = (timer + 1)%100000;
        updateImage();
        checkCollision();
        adjustScroll();
        drawObjects();
        flipEnemies();
        if(stats.getNextSectionFlag()){
            stats.setSection(stats.getSection()+1);
            prepareNextSection();
        }
        if (stats.getDeathFlag()){
            resetCurrentSection();
            stats.setDeathFlag(false);
            player.setState(State.HIT, false);
            
        }

        music.music(stats);
    }


    public void gameStart()
    {
        getBackground().drawImage(new GreenfootImage(gameStart), 0, 0);
        if(Greenfoot.isKeyDown("enter") || Greenfoot.mouseClicked(this)){
            stats.setCurrentLevel(1);
            stats.setGameStartFlag(true);
            player.setUpdatePosition(true);
            prepareNextSection();                       
        }
    }        

    private void gameOver()
    {   
        removeObjects();
        getBackground().drawImage(new GreenfootImage(gameOver), 0, 0);
        music.playGameOver(); 
        player.resetTotal();
        if(Greenfoot.isKeyDown("enter") || Greenfoot.mouseClicked(this)){
            stats.reset();            
            music.stopMusic(); 
            music.setMusicStart(false);            
            stats.setGameOverFlag(false);
            resetCurrentSection();
        }

    }    
    
    public boolean removeExempt(Actor a)
    {
        if (a instanceof MusicController){
            return true;
        }
        if (a instanceof PlayerBullet){
            return true;
        }
        if (a instanceof WorldStats){
            return true;
        }        
        if (a instanceof Collision){
            return true;
        }                       
        if (a instanceof Player){
            a.getImage().setTransparency(0);
            return true;
        }
        return false;
    }
    
    private void removeObjects(){
        List worldObjects = getObjects(Actor.class);
        for (int i = 0; i < worldObjects.size(); i++){
            if (removeExempt((Actor)worldObjects.get(i))){
                continue;
            }
            removeObject((Actor)worldObjects.get(i));
        }        
        
    }
    
    public void prepareNextSection()
    {     
        if(stats.getSection() == stats.getCurrentLevel().numSections()-1){  //boss room
            music.setMusicStart(false);
        }
        collision.setDimensions(stats.getCurrentLevel().getCurrentLogicalImage(stats.getSection()).getWidth(), stats.getCurrentLevel().getCurrentLogicalImage(stats.getSection()).getHeight());
         
        removeObjects(getObjects(Enemy.class));        
        Enemy[] load = stats.getCurrentEnemies();
        if(load != null){
            for(int i=0; i<load.length; i++){
                load[i].getImage().setTransparency(0);                
                addObject(load[i],0,0);
                load[i].reset();
            }
        }
    
        removeObjects(getObjects(Powerup.class));
        Powerup[] powerLoad = stats.getCurrentPowerups();
        if(powerLoad != null){
            for(int i=0; i<powerLoad.length; i++){
                    powerLoad[i].getImage().setTransparency(0);
                    addObject(powerLoad[i],0,0);
            }
        }              
        
        player.setExactX(stats.getCurrentLevel().getSection(stats.getSection()).getPoint().getX());
        player.setExactY(stats.getCurrentLevel().getSection(stats.getSection()).getPoint().getY());
        stats.getCurrentLevel().setXScroll(0);    
    }   
    
    public void resetCurrentSection()
    {           
        music.setMusicStart(false);        
        
        removeObjects(getObjects(Enemy.class));        
        Enemy[] load = stats.getCurrentEnemies();
        if(load != null){
            for(int i=0; i<load.length; i++){
                    load[i].getImage().setTransparency(0);
                    addObject(load[i],0,0);                
                    load[i].reset();
            }
        }
        
        removeObjects(getObjects(Powerup.class));
        Powerup[] powerLoad = stats.getCurrentPowerups();
        if(powerLoad != null){
            for(int i=0; i<powerLoad.length; i++){
                    powerLoad[i].getImage().setTransparency(0);
                    addObject(powerLoad[i],0,0);
            }
        }        
        
        player.setExactX(stats.getCurrentLevel().getSection(stats.getSection()).getPoint().getX());
        player.setExactY(stats.getCurrentLevel().getSection(stats.getSection()).getPoint().getY());
        stats.getCurrentLevel().setXScroll(0);
        player.resetSection();
                
    }
    
    /**
     * Checks objects for wall collisions and calls a function to move them outside walls
     */
    private void checkCollision()
    {
        if(!stats.getDeathFlag() && !player.getState(State.HIT)){
            collision.doPhysics(player);            
        }


        List w = getObjects(Walker.class);
        Walker walker;
        for(int i = 0; i<w.size(); i++){
            walker = ((Walker)w.get(i));
            collision.doPhysics(walker);
        }

        List s = getObjects(Spinner.class);
        Spinner spinner;
        for(int i = 0; i<s.size(); i++){
            spinner = ((Spinner)s.get(i));
            collision.doPhysics(spinner);
        }
    }
    
    private boolean isExempt(Enemy e){      //checks enemy type to see if it is a special exemption from the normal rules
        if(e instanceof Boss1Part)
            return true;
        return false;
    }

    private void adjustScroll(){
        int levelWidth = stats.getCurrentLevel().getCurrentLogicalImage(stats.getSection()).getWidth();              
        int screenWidth = getWidth();
        double x = player.getExactX();

        if(x < screenWidth/2){  //don't scroll. player in first screen
            stats.getCurrentLevel().setXScroll(0);
        }else if(x >=screenWidth/2 && x< levelWidth-screenWidth/2){     //scroll
            stats.getCurrentLevel().setXScroll(getWidth()/2.0 - player.getExactX());
        }else{      //don't scroll. player in last screen
            stats.getCurrentLevel().setXScroll(screenWidth-levelWidth);
        }        
    }
    
    private void drawObjects(){   
        List e = getObjects(Enemy.class);
        Enemy enemy;
        double xScroll = stats.getCurrentLevel().getXScroll();
        double yScroll = stats.getCurrentLevel().getYScroll();        
        for(int i=0; i<e.size(); i++){
            enemy = ((Enemy)e.get(i));
            if( bound(enemy.getExactX()+xScroll, getWidth()) && bound(enemy.getExactY()+yScroll, getHeight()) ){
                enemy.setLocation((int)Math.round(enemy.getExactX()+xScroll), (int)Math.round(enemy.getExactY()+yScroll));
                if(!isExempt(enemy)){
                    enemy.setActive(true);
                }
            }else{
                enemy.setActive(false);                
            }
        }
        
        List b = getObjects(Bullet.class);
        Bullet shoot;
        for(int i=0; i<b.size(); i++){
            shoot = ((Bullet)b.get(i));
            if( bound(shoot.getExactX()+xScroll, getWidth()) && bound(shoot.getExactY()+yScroll, getHeight()) ){
                shoot.setLocation((int)Math.round(shoot.getExactX()+xScroll), (int)Math.round(shoot.getExactY()+yScroll));
                if(!bound(shoot.getX(), getWidth()) || !bound(shoot.getY(), getHeight())){
                    shoot.setActive(false);
                }
            }else{
                if(shoot instanceof ShooterBullet && ((ShooterBullet)shoot).getParentDead()==true){
                    removeObject(shoot);                
                }else{
                    shoot.setActive(false);                
                }
            }
        }  

        List p = getObjects(Powerup.class);
        Powerup powerup;
        for(int i=0; i<p.size(); i++){
            powerup = ((Powerup)p.get(i));
            if( bound(powerup.getExactX()+xScroll, getWidth()) && bound(powerup.getExactY()+yScroll, getHeight()) ){
                powerup.setLocation((int)Math.round(powerup.getExactX()+xScroll), (int)Math.round(powerup.getExactY()+yScroll));
                powerup.setActive(true);
            }else{
                powerup.setActive(false);                
            }
        }       
        
        List ex = getObjects(Explosion.class);
        Explosion explosion;
        for(int i=0; i<ex.size(); i++){
            explosion = ((Explosion)ex.get(i));
            if( bound(explosion.getExactX()+xScroll, getWidth()) && bound(explosion.getExactY()+yScroll, getHeight()) ){
                explosion.setLocation((int)Math.round(explosion.getExactX()+xScroll), (int)Math.round(explosion.getExactY()+yScroll));
                explosion.setActive(true);
            }
        }               
        
    }
    
    private void flipEnemies(){     //flip enemies that are affected by player flip
        List w = getObjects(Walker.class);        
        Walker walker;
        Player p = ((Player)getObjects(Player.class).get(0));
        for(int i=0; i<w.size(); i++){
            walker = ((Walker)w.get(i));
            if(!walker.getState(WalkerState.INVERTED) && walker.getState(WalkerState.FLIPPED) != stats.getFlip()){
                if(walker.getState(WalkerState.CANFLIP)){        
                    walker.setState(WalkerState.FLIP, true);
                }
            }else if(walker.getState(WalkerState.INVERTED) && walker.getState(WalkerState.FLIPPED) == stats.getFlip()){
                if(walker.getState(WalkerState.CANFLIP)){
                    walker.setState(WalkerState.FLIP, true);
                }                
            }
        }   
    }
    
    private boolean bound(double a, int upper){
        if(a >= 0 && a<= upper)
            return true;
        else
            return false;
    }
    
    private void updateImage()
    {
        getBackground().drawImage(stats.getCurrentLevel().getCurrentImage(stats.getSection()), (int)Math.round(stats.getCurrentLevel().getXScroll()) , (int)Math.round(stats.getCurrentLevel().getYScroll()) );            
    }
    
}
