import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
/**
 * The player in the game. This class stores state information for and determines the image of the player.
 * 
 * @author Josh Redpath
 * @version 1.0
 */
public class Player  extends LiveObject
{
    private static final int imageUpdateFrequency = 4;
    private static enum ImageGroup {fireball, flip, stand, run, downGround, upGround, runUp, jump, downAir, upAir, death};
    private static final boolean playSFX = true;
    private static final int maxShootDelay = 1;
    private static final int maxInvincibleDelay = 100;
    
    private int lives;
    private int score;
    private int power;
    private int invincibleTimer;
    private boolean[] state;
    private PlayerInput input;
    private ImageGroup lastImageType;
    private int imageIndex;
    private int shootDelay;
    
    private static GreenfootSound flipSfx;
    private static GreenfootSound flipSfx2;
    private static GreenfootSound flipFire;
    private static GreenfootSound flipFire2;    
    private static GreenfootSound shoot1;
    private static GreenfootSound shoot2;    
    private static GreenfootSound shootWide1;
    private static GreenfootSound shootWide2;        
    private static GreenfootSound armorHit;
    
    private static PlayerBullet[] regBullet = {
                                            new PlayerBullet(),
                                            new PlayerBullet()                                            
                                                };
    private int shootIndex;                                             
    
    private static GreenfootImage[] death = {
                          new GreenfootImage("images/player/death/explode1.png"),
                          new GreenfootImage("images/player/death/explode2.png"),
                          new GreenfootImage("images/player/death/explode3.png"),
                          new GreenfootImage("images/player/death/explode4.png"),
                          new GreenfootImage("images/player/death/explode5.png"),
                          new GreenfootImage("images/player/death/explode6.png"),
                          new GreenfootImage("images/player/death/explode7.png"),
                          new GreenfootImage("images/player/death/explode8.png"),
                          new GreenfootImage("images/player/death/explode9.png"),
                          new GreenfootImage("images/player/death/explode10.png"),
                          new GreenfootImage("images/player/death/explode11.png"),
                          new GreenfootImage("images/player/death/explode12.png"),
                          new GreenfootImage("images/player/death/explode13.png"),
                          new GreenfootImage("images/player/death/explode14.png"),
                          new GreenfootImage("images/player/death/explode15.png"),
                          new GreenfootImage("images/player/death/explode16.png"),
                          new GreenfootImage("images/player/death/explode17.png")
                                                   };
    
    private static final GreenfootImage[] fireball = {
                          new GreenfootImage("images/player/fireball/fireBall1.png"),
                          new GreenfootImage("images/player/fireball/fireBall2.png"),
                          new GreenfootImage("images/player/fireball/fireBall3.png")                          
                                                 };
                                                
    private static final GreenfootImage[] flip = {
                          new GreenfootImage("images/player/air/flip1.png"),
                          new GreenfootImage("images/player/air/flip2.png"),
                          new GreenfootImage("images/player/air/flip3.png"),    
                          new GreenfootImage("images/player/air/flip4.png"),        
                          new GreenfootImage("images/player/air/flip5.png")                          
                                                };
                              
    private static final GreenfootImage stand = new GreenfootImage("images/player/ground/stand1.png");                                                
                          
    private static final GreenfootImage[] run = {
                          new GreenfootImage("images/player/ground/run1.png"),
                          new GreenfootImage("images/player/ground/run2.png"),
                          new GreenfootImage("images/player/ground/run3.png"),    
                          new GreenfootImage("images/player/ground/run4.png"),
                          new GreenfootImage("images/player/ground/run5.png"),
                          new GreenfootImage("images/player/ground/run6.png"),
                          new GreenfootImage("images/player/ground/run7.png"),    
                          new GreenfootImage("images/player/ground/run8.png")                          
                                                };        
    
    private static final GreenfootImage[] downGround = {
                          new GreenfootImage("images/player/ground/down1.png"),        
                          new GreenfootImage("images/player/ground/down2.png")                          
                                                };

    private static final GreenfootImage[] upGround = {
                          new GreenfootImage("images/player/ground/up1.png"),        
                          new GreenfootImage("images/player/ground/up2.png")                          
                                                };                                                
                                                
    private static final GreenfootImage[] runUp = {
                          new GreenfootImage("images/player/ground/runUp1.png"),
                          new GreenfootImage("images/player/ground/runUp2.png"),
                          new GreenfootImage("images/player/ground/runUp3.png"),    
                          new GreenfootImage("images/player/ground/runUp4.png"),
                          new GreenfootImage("images/player/ground/runUp5.png"),
                          new GreenfootImage("images/player/ground/runUp6.png"),
                          new GreenfootImage("images/player/ground/runUp7.png"),    
                          new GreenfootImage("images/player/ground/runUp8.png")                          
                                                };    

    private static final GreenfootImage jump = new GreenfootImage("images/player/air/jump1.png");                                                 
                                                
    private static final GreenfootImage[] downAir = {
                          new GreenfootImage("images/player/air/jumpDown1.png"),        
                          new GreenfootImage("images/player/air/jumpDown2.png")                          
                                                };

    private static final GreenfootImage[] upAir = {
                          new GreenfootImage("images/player/air/jumpUp1.png"),        
                          new GreenfootImage("images/player/air/jumpUp2.png")                          
                                                };                                   

              
    
    public Player(int x, int y)
    {
        super(x,y);
        input = new PlayerInput();
        invincibleTimer = 0;
        lives = 3;
        score = 0;
        power = Power.NONE;
        actTimer = 1;
        lastImageType = ImageGroup.stand;
        imageWidth = stand.getWidth();
        imageHeight = stand.getHeight();
        imageIndex = 0;
        shootIndex = 0;
        shootDelay = maxShootDelay;
        dx = 1.3;
        //dx = 2;        
        dy = 0;
        baseDy = 1;        
        try{
            flipSfx = new GreenfootSound("sounds/flipSfx.mp3");
            flipSfx2 = new GreenfootSound("sounds/flipSfx.mp3");   
        }catch(Exception e){}
        try{
            flipFire = new GreenfootSound("sounds/flip2.mp3");
            flipFire2 = new GreenfootSound("sounds/flip2.mp3");   
        }catch(Exception e){}
        try{
            shoot1 = new GreenfootSound("sounds/playerShootNormal.mp3");     
            shoot2 = new GreenfootSound("sounds/playerShootNormal.mp3");                 
        }catch(Exception e){}
        try{
            shootWide1 = new GreenfootSound("sounds/wideShot.mp3");     
            shootWide2 = new GreenfootSound("sounds/wideShot.mp3");                 
        }catch(Exception e){}        
        try{
            armorHit = new GreenfootSound("sounds/armorHit.mp3");              
        }catch(Exception e){}
        
        state = new boolean[23];
        initializePlayerState();
        
    }
    
    public void resetSection(){
        setImage(new GreenfootImage(stand));        
        invincibleTimer = 0;
        actTimer = 1;
        lastImageType = ImageGroup.stand;
        imageIndex = 0;
        shootIndex = 0;
        shootDelay = maxShootDelay;  
        dy = 0;
        updatePosition = false;
        getImage().setTransparency(0);        
        initializePlayerState();
    }    
    
    public void resetTotal(){
        setImage(new GreenfootImage(stand));        
        invincibleTimer = 0;
        lives = 3;
        actTimer = 1;
        lastImageType = ImageGroup.stand;
        imageIndex = 0;
        shootIndex = 0;
        shootDelay = maxShootDelay;  
        dy = 0;
        updatePosition = false;
        state[State.HIT] = false;
        getImage().setTransparency(0);
        initializePlayerState();
    }        
    
    /**
     * Sets the state variables for the player as they should be at level start
     */
    public void initializePlayerState()
    {
        //player starts on the ground with no armor or powerups facing to the right
        state[State.MOVING] = false;
        state[State.LEFT] = false;        
        state[State.RIGHT] = true;
        state[State.DOWN] = false;
        state[State.UP] = false;
        state[State.JUMP] = false;
        state[State.SHOOT] = false;
        state[State.GROUND] = true;
        state[State.ARMOR] = false;
        state[State.INVINCIBLE] = false;
        state[State.FLIPPED] = false;    
        state[State.CEILING] = false;
        state[State.RWALL] = false;
        state[State.LWALL] = false; 
        state[State.CROUCH] = false;
        state[State.CANFLIP] = false;        
        state[State.INWALL] = false;
        state[State.READYJUMP] = false;        
        state[State.GROUNDSP] = false;
        state[State.CEILINGSP] = false;        
        state[State.CANJUMP] = false;
        state[State.FIREBALL] = false;
    }
    
    /**
     * Used for testing states during gameplay
     */
    private void printState()
    {
        System.out.println("Moving: "+state[State.MOVING]);
        System.out.println("Left: "+state[State.LEFT]);
        System.out.println("Right: "+state[State.RIGHT]);
        System.out.println("Down: "+state[State.DOWN]);
        System.out.println("Up: "+state[State.UP]);
        System.out.println("Jump: "+state[State.JUMP]);
        System.out.println("Shoot: "+state[State.SHOOT]);
        System.out.println("Ground: "+state[State.GROUND]);
        System.out.println("Armor: "+state[State.ARMOR]);
        System.out.println("Invincible: "+state[State.INVINCIBLE]);
        System.out.println("Flipped: "+state[State.FLIPPED]);     
        
        System.out.println("Ceiling: "+state[State.CEILING]);
        System.out.println("Right Wall: "+state[State.RWALL]);
        System.out.println("Left Wall: "+state[State.LWALL]);   
        System.out.println("Crouch: "+state[State.CROUCH]);
        System.out.println("CanFlip: "+state[State.CANFLIP]);        
        System.out.println("InWall: "+state[State.INWALL]);          
        System.out.println("ReadyJump: "+state[State.READYJUMP]);        
        System.out.println("GroundSP: "+state[State.GROUNDSP]);
        System.out.println("CeilingSP: "+state[State.CEILINGSP]);        
        System.out.println("CanJump: "+state[State.CANJUMP]);  
        System.out.println("Hit: "+state[State.HIT]);             
        System.out.println("Fireball: "+state[State.FIREBALL]);             
        System.out.println("");
    }    
    
    public int getPower(){
        return power;    
    }
    public void setPower(int p){
        power = p;    
    }
    
    public void addedToWorld(World world){
        for(int i=0; i<regBullet.length; i++){
            world.addObject(regBullet[i],0,0);
        }       
    }
    
    public void playFlip()
    {
            if(!playSFX){
                return;
            }
            if(power == Power.FIREBALL){
                if(flipFire != null && flipFire.isPlaying() == false){
                    try{flipFire.play();}catch(Exception e){}
                }else{
                    try{flipFire2.play();}catch(Exception e){}              
                }                                
            }else{
                if(flipSfx != null && flipSfx.isPlaying() == false){
                    try{flipSfx.play();}catch(Exception e){}
                }else{
                    try{flipSfx2.play();}catch(Exception e){}              
                }                
            }   
    }    
    
    public void playShoot()
    {
            if(!playSFX){
                return;
            }
            if(power == Power.GUN){
                if(shootWide1 != null && shootWide1.isPlaying() == false){
                    try{shootWide1.play();}catch(Exception e){}
                }else{
                    try{shootWide2.play();}catch(Exception e){}              
                }                       
            }else{
                if(shoot1 != null && shoot1.isPlaying() == false){
                    try{shoot1.play();}catch(Exception e){}
                }else{
                    try{shoot2.play();}catch(Exception e){}              
                }                
            }   
    }        
    

    
    public void setState(int s, boolean b){
        state[s] = b;
    }
    public boolean getState(int s){
        return state[s];
    }   
    
    private GreenfootImage downGround()
    {
            if(lastImageType == ImageGroup.downGround){
                if(imageIndex+1 == downGround.length)
                    return downGround[imageIndex];
                else{
                    imageIndex++;    
                    return downGround[imageIndex];                    
                }
            }else{
                lastImageType = ImageGroup.downGround;                
                imageIndex = 0;
                return downGround[imageIndex];                                    
            }
    }
    
    private GreenfootImage upGround()
    {
            if(lastImageType == ImageGroup.upGround){
                if(imageIndex+1 == upGround.length)
                    return upGround[imageIndex];
                else{
                    imageIndex++;    
                    return upGround[imageIndex];                    
                }
            }else{
                lastImageType = ImageGroup.upGround;                
                imageIndex = 0;
                return upGround[imageIndex];                                    
            }
    }    
    
    private GreenfootImage downAir()
    {
            if(lastImageType == ImageGroup.downAir){
                if(imageIndex+1 == downAir.length)
                    return downAir[imageIndex];
                else{
                    imageIndex++;    
                    return downAir[imageIndex];                    
                }
            }else{
                lastImageType = ImageGroup.downAir;                
                imageIndex = 0;
                return downAir[imageIndex];
            }
    }    
    
    private GreenfootImage upAir()
    {
            if(lastImageType == ImageGroup.upAir){
                if(imageIndex+1 == upAir.length)
                    return upAir[imageIndex];
                else{
                    imageIndex++;    
                    return upAir[imageIndex];                    
                }
            }else{
                lastImageType = ImageGroup.upAir;                
                imageIndex = 0;
                return upAir[imageIndex];
            }
    }     
    
    private GreenfootImage runUp()
    {
            if(lastImageType == ImageGroup.runUp){
                imageIndex = (imageIndex + 1)%runUp.length;    
                return runUp[imageIndex];                    
            }else{
                lastImageType = ImageGroup.runUp;                
                imageIndex = 0;
                return runUp[imageIndex];                                    
            }
    }   
    
    private GreenfootImage run()
    {
            if(lastImageType == ImageGroup.run){
                imageIndex = (imageIndex + 1)%run.length;    
                return run[imageIndex];                    
            }else{
                lastImageType = ImageGroup.run;                
                imageIndex = 0;
                return run[imageIndex];                                    
            }
    }   
    
    private GreenfootImage fireball()
    {
            if(lastImageType == ImageGroup.fireball){
                imageIndex = imageIndex + 1;
                if(imageIndex >= fireball.length){
                    imageIndex = fireball.length-2;    
                }
                return fireball[imageIndex];                    
            }else{
                lastImageType = ImageGroup.fireball;
                imageIndex = 0;
                return fireball[imageIndex];                                    
            }
    }       
    
    private GreenfootImage stand()
    {
        lastImageType = ImageGroup.stand;
        return stand;
    }
    
    private GreenfootImage jump()
    {
        lastImageType = ImageGroup.jump;
        return jump;
    }    
    
    
//    private static enum ImageGroup {fireball, flip, stand, run, downGround, upGround, runUp, jump, downAir, upAir, death};    
    
    /**
     * Sets the correct player image based on the player's state
     */
    public void setPlayerImage()
    {                
        if(state[State.FLIPPED]){
            if(state[State.CEILING] || state[State.CEILINGSP]){    //on ground
                if(state[State.MOVING]){    //moving
                    if(state[State.UP]){  //aiming down
                        setImage(new GreenfootImage(downGround())); 
                    }else if(state[State.DOWN]){ //aiming up while running
                        setImage(new GreenfootImage(runUp()));                         
                    }else{                  //aiming to the side
                        setImage(new GreenfootImage(run())); 
                    }
                }else{              //not moving
                    if(state[State.UP]){  //aiming down
                        setImage(new GreenfootImage(downGround())); 
                    }else if(state[State.DOWN]){ //aiming up
                        setImage(new GreenfootImage(upGround()));                         
                    }else{                  //standing still
                        setImage(new GreenfootImage(stand())); 
                    }
                }
            }else{      //in air
                if(state[State.FIREBALL]){
                    setImage(new GreenfootImage(fireball())); 
                }else if(state[State.UP]){  //aiming down while in air
                    setImage(new GreenfootImage(downAir())); 
                }else if(state[State.DOWN]){ //aiming up while in air
                    setImage(new GreenfootImage(upAir()));                         
                }else{                  //default air posture
                    setImage(new GreenfootImage(jump())); 
                }
            }                
        }else{
            if(state[State.GROUND] || state[State.GROUNDSP]){    //on ground
                if(state[State.MOVING]){    //moving
                    if(state[State.DOWN]){  //aiming down
                        setImage(new GreenfootImage(downGround())); 
                    }else if(state[State.UP]){ //aiming up while running
                        setImage(new GreenfootImage(runUp()));                         
                    }else{                  //aiming to the side
                        setImage(new GreenfootImage(run())); 
                    }
                }else{              //not moving
                    if(state[State.DOWN]){  //aiming down
                        setImage(new GreenfootImage(downGround())); 
                    }else if(state[State.UP]){ //aiming up
                        setImage(new GreenfootImage(upGround()));                         
                    }else{                  //standing still
                        setImage(new GreenfootImage(stand())); 
                    }
                }
            }else{      //in air
                if(state[State.FIREBALL]){
                    setImage(new GreenfootImage(fireball())); 
                }else if(state[State.DOWN]){  //aiming down while in air
                    setImage(new GreenfootImage(downAir())); 
                }else if(state[State.UP]){ //aiming up while in air
                    setImage(new GreenfootImage(upAir()));                         
                }else{                  //default air posture
                    setImage(new GreenfootImage(jump())); 
                }
            }            
        }
        
        if(state[State.LEFT]){     //use normal image
            getImage().mirrorHorizontally();
        }
        
        if(state[State.FLIPPED]){
            getImage().mirrorVertically();
        }

        if(state[State.INVINCIBLE]){
            getImage().setTransparency(128);
        }

    }
    
    private void updatePosition(){
        if(updatePosition){
            state = input.updatePlayer(state, this);            
            if(getWorld().getObjects(Level.class).size() > 0){

                WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));                
                Level l = stats.getCurrentLevel();
                if(l != null){
                    int levelWidth = l.getCurrentLogicalImage(stats.getSection()).getWidth();              
                    int screenWidth = getWorld().getWidth();
                    double x = getExactX();
                   /* if(getExactX() > l.getCurrentLogicalImage(stats.getSection()).getWidth() - getWorld().getWidth()/2.0){
                        setLocation(  (int)Math.round(getWorld().getWidth()/2.0 + getExactX()-l.getXScroll()), (int)Math.round(getExactY()));          
                    }else{
                        setLocation(Math.min(getWorld().getWidth()/2, (int)Math.round(getExactX())), (int)Math.round(getExactY()));                          
                    }*/
                    if(x < screenWidth/2){  //don't scroll. player in first screen
                        setLocation((int)Math.round(getExactX()), (int)Math.round(getExactY()));
                    }else if(x >=screenWidth/2 && x< levelWidth-screenWidth/2){     //scroll
                        setLocation(screenWidth/2, (int)Math.round(getExactY()));
                    }else{      //don't scroll. player in last screen
                        setLocation((int)Math.round(getExactX()+l.getXScroll()), (int)Math.round(getExactY()));
                    } 
                }
                stats.setFlip(getState(State.FLIPPED));
            }
        }
    }
    
    private double boundFix(double val, int upperBound){
        if(val < 0){
            return 0;    
        }
        if(val > upperBound){
            return upperBound;    
        }
        return val;
    }    
    
    private void checkShoot(){
        if(shootDelay > 0){
            shootDelay--;    
        }
        
        if(state[State.SHOOT] && shootDelay==0){    //if the player is trying to shoot and enough time has passed
            if(regBullet[shootIndex].getActive() == false){     //and the bullet isn't already being fired
                WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));
                GreenfootImage i = stats.getCurrentLevel().getCurrentLogicalImage(stats.getSection());

                int speed = regBullet[shootIndex].getSpeed();
                if(state[State.UP]){
                    regBullet[shootIndex].setImageState(State.UP, power==Power.GUN);
                    regBullet[shootIndex].setExactX(boundFix(getExactX(), i.getWidth()));                    
                    regBullet[shootIndex].setExactY(boundFix(getExactY()-imageHeight/2, i.getHeight()));
                    regBullet[shootIndex].setLocation(getX(),getY()-imageHeight/2);                    
                    regBullet[shootIndex].setDy(-1*speed);
                    regBullet[shootIndex].setDx(0);                    
                }else if(state[State.DOWN]){
                    regBullet[shootIndex].setImageState(State.DOWN, power==Power.GUN);
                    regBullet[shootIndex].setExactX(boundFix(getExactX(), i.getWidth()));                    
                    regBullet[shootIndex].setExactY(boundFix(getExactY()+imageHeight/2,i.getHeight()));
                    regBullet[shootIndex].setLocation(getX(),getY()+imageHeight/2);                    
                    regBullet[shootIndex].setDy(speed);
                    regBullet[shootIndex].setDx(0);                                        
                }else if(state[State.RIGHT]){
                    regBullet[shootIndex].setImageState(State.RIGHT, power==Power.GUN);
                    if(state[State.FLIPPED]){
                        regBullet[shootIndex].setExactY(boundFix(getExactY()+5,i.getHeight()));
                        regBullet[shootIndex].setLocation(getX()+imageWidth/2-2,getY()+5);                        
                    }else{
                        regBullet[shootIndex].setExactY(boundFix(getExactY()-5,i.getHeight()));
                        regBullet[shootIndex].setLocation(getX()+imageWidth/2-2,getY()-5);                        
                    }
                    regBullet[shootIndex].setExactX(boundFix(getExactX()+imageWidth/2-2, i.getWidth()));                    

                    regBullet[shootIndex].setDy(0);
                    regBullet[shootIndex].setDx(speed);                                                            
                }else if(state[State.LEFT]){
                    regBullet[shootIndex].setImageState(State.LEFT, power==Power.GUN);
                    if(state[State.FLIPPED]){
                        regBullet[shootIndex].setExactY(boundFix(getExactY()+5, i.getHeight()));
                        regBullet[shootIndex].setLocation(getX()-imageWidth/2+2,getY()+5);
                    }else{
                        regBullet[shootIndex].setExactY(boundFix(getExactY()-5, i.getHeight()));
                        regBullet[shootIndex].setLocation(getX()-imageWidth/2+2,getY()-5);                        
                    }
                    regBullet[shootIndex].setExactX(boundFix(getExactX()-imageWidth/2+2, i.getWidth()));

                    regBullet[shootIndex].setDy(0);
                    regBullet[shootIndex].setDx(-1*speed);
                }

                regBullet[shootIndex].setActive(true);
                shootDelay = maxShootDelay;
                playShoot();
            }
            shootIndex = (shootIndex+1)%regBullet.length;
        }
        
    }
    
    private boolean enemyCollisionExempt(Enemy e){
            if(e instanceof Boss1Overlay){
                return true;    
            }
        return false;
    }

    private void die(){
        MusicController m = ((MusicController)getWorld().getObjects(MusicController.class).get(0));
        if(imageIndex < death.length){
            if(actTimer%(imageUpdateFrequency+1) == 0){
                if(!m.isExplodePlaying()){
                    m.playExplode();
                }
                setImage(new GreenfootImage(death[imageIndex]));
                imageIndex++;
            }
        }else{
            if (m.isExplodePlaying())
                 return;
                 
            WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));
            stats.setDeathFlag(true);
            imageIndex = 0;
            lives--;

            if (lives <= 0)
            {
                stats.setGameOverFlag(true);
            }
            
        }
    }

    private void checkEnemyCollision(){
        if(state[State.INVINCIBLE])
            return;

        List l = getWorld().getObjects(Enemy.class);
        Enemy e;
        for(int i=0; i<l.size(); i++){
            e = (Enemy)l.get(i);
            if(!enemyCollisionExempt(e) && e.getActive()){
                if(this.intersects(e)){
                    if(state[State.FIREBALL]){  //hurt enemy
                        e.setHealth(e.getHealth()-5);
                        e.setInvert(true);
                        if(e.getHealth() <=0 ){
                            e.kill();    
                        }else{
                            ((MusicController)getWorld().getObjects(MusicController.class).get(0)).playHitEnemy();
                        }
                    }else{
                        if(state[State.ARMOR]){
                            state[State.ARMOR] = false;
                            state[State.INVINCIBLE] = true;
                            invincibleTimer = maxInvincibleDelay;
                            try{armorHit.play();}catch(Exception exception){}
                        }else{
                            state[State.HIT] = true;
                            imageIndex = 0;
                        }                        
                    }

                }
            }
        }

        List m = getWorld().getObjects(Bullet.class);
        Bullet b;
        for(int i=0; i<m.size(); i++){
            b = (Bullet)m.get(i);
            if(this.intersects(b) && b.getActive() && !(b instanceof PlayerBullet)){
                    if(state[State.FIREBALL]){
                        //ignore bullet
                    }else{
                        if(state[State.ARMOR]){
                            state[State.ARMOR] = false;
                            state[State.INVINCIBLE] = true;
                            invincibleTimer = maxInvincibleDelay;
                        }else{
                            state[State.HIT] = true;
                            imageIndex = 0;
                        }                        
                    }

            }
        }

    }

    
    
    private void checkPowerup(){
        List w = getWorld().getObjects(Powerup.class);
        for(int i=0; i<w.size(); i++){
            if(intersects((Powerup)w.get(i))){
                if((Powerup)w.get(i) instanceof Armor){
                    state[State.ARMOR] = true;    
                }else{
                    power = ((Powerup)w.get(i)).getType();    
                }
                ((MusicController)getWorld().getObjects(MusicController.class).get(0)).playGetPowerup();
                getWorld().removeObject((Powerup)w.get(i));
            }
        }
    }
    
    public int getInvincibleTimer(){
        return invincibleTimer;
    }
    public void setInvincibleTimer(int i){
        invincibleTimer = i;
    }



    /**
     * Act - do whatever the Player wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        actTimer = (actTimer + 1)%100000;      
        if(invincibleTimer > 0){
            invincibleTimer--;
            if(invincibleTimer == 0){
                state[State.INVINCIBLE] = false;
            }
        }        
        
        if(state[State.HIT]){
            die();
        }else{
            WorldStats stats = ((WorldStats)getWorld().getObjects(WorldStats.class).get(0));
            if(actTimer%imageUpdateFrequency == 0 && !stats.getGameOverFlag() && stats.getGameStartFlag()){
                setPlayerImage();
            }
            updatePosition();
            checkShoot();
            
            if(!stats.getDeathFlag()){
                checkEnemyCollision();
                checkPowerup();
            }
        }


        if(actTimer%8 == 0){
            //System.out.println(getExactX()+","+getExactY());
        }
    }    
}

