import greenfoot.*;

/**
 * Write a description of class PlayerInput here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class PlayerInput  extends MoveBehavior 
{
   private static final double gravity = 0.22;
   private static final double jumpAccel = 0.16;
   private static final double maxSpeed = 3;
   private static final int jumpCount = 6;      //determines the length of time a jump is stored
   private static final int scaleJump = 4;      //multiplies jumpCount to determine length of time a jump is
   private int jumpCounter;

   public PlayerInput()
   {
        super(1);
        jumpCounter = 0;
   } 
    
   /**
   * Updates the player's state variables based on player input
   */      
    public boolean[] updatePlayer(boolean[] state, Player p)
    {  
        state = processHorizontalState(state,p);
        state = processVerticalState(state,p);
        state = checkCeilingHit(state,p);
        state = checkGroundHit(state,p);
        state = resetFlip(state,p);
        state = prepareJump(state, p);
        state = checkShoot(state,p);
        state = processJump(state,p);
        state = checkFlip(state,p);
        
        return state;
    }    
    
   private boolean[] processVerticalState(boolean[] state, Player p)
   {
        if(state[State.FLIPPED]){
            if(Greenfoot.isKeyDown(Input.DOWN)){              //check if the player is holding up or down           
                state[State.UP] = false;
                state[State.DOWN] = true; 
                state[State.CROUCH]= false;            
            }else if(Greenfoot.isKeyDown(Input.UP)){          
                state[State.UP] = true;
                state[State.DOWN] = false;   
                if(state[State.CEILING] == true || state[State.CEILINGSP]){
                    state[State.CROUCH]=true;
                }else{
                    state[State.CROUCH]= false;            
                }
            }else{
                state[State.UP] = false;
                state[State.DOWN] = false;             
                state[State.CROUCH]= false;            
            }             
        }else{
            if(Greenfoot.isKeyDown(Input.UP)){              //check if the player is holding up or down           
                state[State.UP] = true;
                state[State.DOWN] = false; 
                state[State.CROUCH]= false;            
            }else if(Greenfoot.isKeyDown(Input.DOWN)){          
                state[State.UP] = false;
                state[State.DOWN] = true;   
                if(state[State.GROUND] == true || state[State.GROUNDSP]){
                    state[State.CROUCH]=true;
                }else{
                    state[State.CROUCH]= false;            
                }
            }else{
                state[State.UP] = false;
                state[State.DOWN] = false;             
                state[State.CROUCH]= false;            
            }            
        }
        return state;
   }
    
   private boolean[] processHorizontalState(boolean[] state, Player p)
   {
        if(Greenfoot.isKeyDown(Input.LEFT)){            //check if the player is moving left or right
            if(state[State.LWALL] == false && state[State.CROUCH] == false)            
                p.setExactX(p.getExactX()-p.getDx());
            
            state[State.LEFT] = true;
            state[State.RIGHT] = false;        
            state[State.MOVING] = true; 
        }else if(Greenfoot.isKeyDown(Input.RIGHT)){
            if(state[State.RWALL] == false && state[State.CROUCH] == false)            
                p.setExactX(p.getExactX()+p.getDx());
            
            state[State.LEFT] = false;
            state[State.RIGHT] = true;                      
            state[State.MOVING] = true; 
        }else{
            state[State.MOVING] = false;             
        }
       return state;
   }
    
   private boolean[] prepareJump(boolean[] state, Player p)
   {
        if(state[State.CANJUMP] && jumpCounter < jumpCount && state[State.READYJUMP] == false){
                if(Greenfoot.isKeyDown(Input.JUMP) || (jumpCounter > 0 && (jumpCounter != jumpCount/6) && (jumpCounter != 2*jumpCount/3)) ){
                    jumpCounter++;    
                }else{
                    state[State.READYJUMP] = true;                 
                    jumpCounter = jumpCounter * scaleJump;
                }
        }else{
            if(state[State.READYJUMP] == false){
                jumpCounter = jumpCounter * scaleJump;                               
            }   
            state[State.READYJUMP] = true;           
        }
       return state;
   }
    
   private boolean[] checkFlip(boolean[] state, Player p)    
   {
        if(state[State.CANFLIP]){           //if it is possible for player to flip gravity
            if(state[State.FLIPPED]){   //player is already upside down
                if(Greenfoot.isKeyDown(Input.JUMP) && state[State.DOWN]){
                    state[State.FLIPPED] = false;
                    state[State.CANFLIP] = false;
                    state[State.CANJUMP] = false;                                        
                    state[State.READYJUMP] = true;
                    if(p.getPower() == Power.FIREBALL){
                        state[State.FIREBALL] = true;
                    }
                    p.playFlip();
                    //p.setDy(-1*p.getDy());
                    p.setDy(0);                                        
                    jumpCounter = 0;
                }    
            }else{                      //player is right side up
                if(Greenfoot.isKeyDown(Input.JUMP) && state[State.UP]){
                    state[State.FLIPPED] = true;
                    state[State.CANFLIP] = false;
                    state[State.CANJUMP] = false;                    
                    state[State.READYJUMP] = true;    
                    if(p.getPower() == Power.FIREBALL){
                        state[State.FIREBALL] = true;
                    }                    
                    p.playFlip();
                    //p.setDy(-1*p.getDy());                    
                    p.setDy(0);                    
                    jumpCounter = 0;
                }            
            }
        } 
        return state;
   }
    
   private boolean[] processJump(boolean[] state, Player p)
   {
        if( (state[State.READYJUMP] && jumpCounter > 0) ){            //check if the player is pressing jump
            state[State.CANJUMP] = false;                     
            jumpCounter--;
            state[State.JUMP] = true;   
            if(state[State.FLIPPED]){
                if(state[State.CEILING] || state[State.CEILINGSP]){        //first jump off ground
                    p.resetDy();
                    p.setExactY(p.getExactY()+p.getDy());
                    p.setDy(p.getDy()+jumpAccel*jumpCounter);
                }else{                                  //continued jump
                    p.setExactY(p.getExactY()+p.getDy());
                    p.setDy(p.getDy()-jumpAccel);          
                }                 
            }else{
                if(state[State.GROUND] == true || state[State.GROUNDSP]){        //first jump off ground
                    p.resetDy();
                    p.setExactY(p.getExactY()-p.getDy());
                    p.setDy(p.getDy()+jumpAccel*jumpCounter);
                }else{                                  //continued jump
                    p.setExactY(p.getExactY()-p.getDy());
                    p.setDy(p.getDy()-jumpAccel);          
                }                
            }

        }else{                                          //otherwise the player is falling
            //jumpCounter = 0;
            if(state[State.JUMP] == true){   //previously jumping
                p.setDy(0);
            }
            state[State.JUMP] = false;  
            if(state[State.FLIPPED]){
                if(!state[State.CEILING] && !state[State.CEILINGSP]){   //if still in air
                    p.setDy(p.getDy()+gravity);                  
                    p.setExactY(p.getExactY()-p.getDy());
                }else{          //at ground, reset counter        
                }                 
            }else{
                if(state[State.GROUND] == false && state[State.GROUNDSP] == false){   //if still in air
                    p.setDy(p.getDy()+gravity );                  
                    p.setExactY(p.getExactY()+p.getDy());
                }else{          //at ground, reset counter
                }                
            }

        }  
       return state;
   }
    
   private boolean[] checkShoot(boolean[] state, Player p)
   {
        if(Greenfoot.isKeyDown(Input.SHOOT)){           //check if the player is shooting
            state[State.SHOOT] = true;                  
        }else{
            state[State.SHOOT] = false;                              
        }    
        return state;
   }
    
   private boolean[] checkCeilingHit(boolean[] state, Player p)
   {
        if(state[State.FLIPPED]){
            if(state[State.GROUND] || state[State.GROUNDSP]){
                jumpCounter = 0;
            }
        }else{
            if(state[State.CEILING] || state[State.CEILINGSP]){
                jumpCounter = 0;
            }
        }  
        return state;
   }    
    
   private boolean[] checkGroundHit(boolean[] state, Player p)
   {
        if(!state[State.FLIPPED]){
            if(state[State.GROUND] || state[State.GROUNDSP]){
                if( state[State.READYJUMP] == true){
                    jumpCounter = 0;    
                }
                state[State.READYJUMP] = false;
                state[State.FIREBALL] = false;                
            }
        }else{
            if(state[State.CEILING] || state[State.CEILINGSP]){
                if( state[State.READYJUMP] == true){
                    jumpCounter = 0;    
                }                
                state[State.READYJUMP] = false;     
                state[State.FIREBALL] = false;                
            }
        }     
        return state;
   }
    
   private boolean[] resetFlip(boolean[] state, Player p)
   {
        if(!state[State.FLIPPED] && (state[State.GROUND] || state[State.GROUNDSP]) && jumpCounter == 0 ){
                if(Greenfoot.isKeyDown(Input.JUMP)){
                    
                }else{
                    state[State.CANFLIP] = true;
                    state[State.CANJUMP] = true;                
                }
        }else if(state[State.FLIPPED] && (state[State.CEILING] || state[State.CEILINGSP]) && jumpCounter == 0 ){
                if(Greenfoot.isKeyDown(Input.JUMP)){
                    
                }else{
                    state[State.CANFLIP] = true;                    
                    state[State.CANJUMP] = true;                
                }         
        }
        return state;
   }
}
